/***************************************************************

		motor.h

   This file contains the #defines, structures, and 
   extern declarations for the file motor.c

***************************************************************/


#define NUMBER_AXIS  3

#define DISABLE_ALL "F"			/* special strings for drives */
#define ENABLE_ALL  "E"
#define DRIVES_OFF  "2F 4F"
#define RESET_DRIVES "2E 4E 2Z 4Z"

#define ON		1
#define OFF		0

#define X_ID            1               /*  compumotor indexer id #s */
#define Y_ID            3
#define Z_ID            5

#define X_ord           0               /*  axis ordinals */
#define Y_ord           1
#define Z_ord           2

#define STD_ord         3


#define RESET_TIME	12
#define INIT_TIME	4       /*  time to reset indexers */
#define HOMETIME	60	/*  time to wait while homing */
#define MOVETIME	15	/*  time to wait while moving */
#define MAXCHARTIME	100	/*  time to wait on char read in 1/10s sec. */
#define TIME_CUSHION	7.0	/*  safety margin on timeouts */

#define ENCODER_RESOLUTION 2032 


#define ZINITTIME	6		/* time is sec to reset z axis */
#define BITTIME		0.05		/* toggle time for a command bit */
#define RELAXTIME       0.10		/* time gripper takes to relax */
#define CLOSETIME       0.40		/* time gripper takes to close */

#define MAX_Z_IF_MOVING  1000

#define	Z_GRAB_VEL	40.0		/* speed when grabbing a button */
#define Z_GRAB_ACCEL	20.0		/* acceleration when grabbing button*/
#define CRUISE_HEIGHT   250		/* height at which buttons are carried*/
					/* old value was 500 */
#define ZHILIMITFORCE  	60000		/* z value that will reach limit first*/
#define ZPARK		10000
#define OPEN		0		/* jaws are open */
#define CLOSED		1		/* jaws are closed */
#define RELAXED		2		/* jaws are relaxed */
#define SHORT		1		/* gripper motor stroke flags */
#define LONG		4

#define LOWHOMEACCEL	.5		/* for creeping into home */
#define LOWHOMESPEED	.5
#define HIHOMEACCEL	5.0		/* for searching for home */
#define HIHOMESPEED	5.0
#define ZHOMESPEED	1.5
#define HIGH_VEL	45.0		/* default high velocity */
#define HIGH_ACCEL	500.0		/* default high acceleration */
#define CARRY_VEL	45.0		/* velocity for carrying buttons */
#define CARRY_ACCEL	500.0		/* acceleration for carrying buttons */
#define BUMP_VEL	20.0		/* used when "nudging" gripper */
#define BUMP_ACCEL	20.0

#define SYNC		1		/* Z synchronizes with software */
#define NOSYNC		0		/* Z does not sync with software */

#define XOFFSET		16040		/* old value was = 16257 */
					/* "home" to plate center */
#define YOFFSET		-26790          /* old value was = -26711 */
					/* "home" to plate center */
#define ZOFFSET		470		/* to insure we get off of "edge" */
					/* old value was 940 */

#define STOW_X		89500		/*89950 coordinates of gripper when stowed */
#define STOW_Y	       -89000

#define SCREEN_X_IN     93500
#define SCREEN_X_OUT   -95300
#define SCREEN_Y       -87800
#define SCREEN_Z        9000
#define XY_TOLERANCE    80

#define RETRACTED	0		/* states of calibration screen */
#define PROBE_UP	1
#define EXTENDING	2
#define DEPLOYED	3

#define VOID_COORD	999999

#define NUDGEOFFHOME    3000		/* distance to force axis off home */

#define BUTTON		0
#define SHAFT 		1	/* array indices for trigger bits on Z */
#define ZBELOWHOME      2

#define XY_OK		0
#define XY_LIM		1
#define XY_FAIL		2

#define PLUNGER_IN	1
#define PLUNGER_OUT	0

#define CLOSE_TRYS	10	/* number of attempts at closing gripper*/

#define NORTH		-1	/*  direction of table y motion WRT sky */
#define EAST		 1 	/*  direction of table x motion WRT sky */

#define PHYSICAL	0	/*  flag values for coord sys toggle */
#define VIRTUAL		1

#define MAXTAGS		50	/* number of position markers */

#define SEC_TO_USEC	1000000.0


/****************** functions ********************/

extern void tagCoord();
extern void defTag();
extern void printTag();
extern int  WhamMotors();
extern int  InitMotors();
extern void InitDrives();
extern int  HomeMotors();
extern int  send_it();
extern void gripOut();
extern void gripIn();
extern void gripTest();
extern void Open();
extern void OpenWide();
extern void OpenAbsolute();
extern void Close();
extern void CloseWide();
extern void CloseAbsolute();
extern void Relax();
extern void SetStroke();
extern void Hold();
extern int  Brakes();
extern int  Zlight();
extern void gripIn();
extern void gripOut();
extern void screenIn();
extern void screenOut();
extern void screenRecover();
extern int  XYokToMove();
extern int  EraseProgram();
extern int  BeginProgram();
extern int  EndProgram();
extern int  ExecProgram();
extern int  ProgDelay();
extern void secToTimer();
extern void syncToIndexers();

extern char*  ordToName();
extern double calcMoveTime();

/*************** Globals ****************/


extern struct axis_characteristics axis[]; /* contains physical limits */

extern struct plate_coords location_now;  /* Where the gripper is */

extern struct plate_coords Tags[]; 	 /* location markers for convenience*/

extern int ScreenOverride;		/* allow x,y motion with Z retracted */

extern int RestrictedMoves;	         /*  force additional error checking */
					 /*  if true. */
extern int GripPos;			 /* last known status of gripper */

extern int MoveInProgress;		 /* used by error recovery */

extern int GripperParked;		/* prevents x,y motion when true */

extern int ScreenStatus;		/* state of the calibration screen */

/*************  Structures **************/

struct  long_limits{
	long min;
	long max;
	long dflt;
	};


struct flt_limits{
	double min;
	double max;
	double dflt;
	};


struct axis_characteristics{
	int unitID;
	double accel_now;
	double velocity_now;
	struct flt_limits accel;
	struct flt_limits velocity;
	struct long_limits position;
	};

struct plate_coords{                /*  encoder steps */
	long x;
	long y;
	long z;
	};


struct axisListNode{
	int axisOrd;
	struct axisListNode *next;
	};


